MODULE traldf_iso_grif
   !!==============================================================================
   !!                   ***  MODULE  traldf_iso_grif  ***
   !! Ocean active tracers:  horizontal component of the lateral tracer mixing trend
   !!==============================================================================
   !! History :   9.0  !  06-10  (C. Harris)
   !!----------------------------------------------------------------------
#if   defined key_ldfslp   ||   defined key_esopa
   !!----------------------------------------------------------------------
   !!   'key_ldfslp'               slope of the lateral diffusive direction
   !!----------------------------------------------------------------------
   !!   tra_ldf_iso_grif  : update the tracer trend with the horizontal 
   !!                       component of a iso-neutral laplacian operator
   !!                       and with the vertical part of
   !!                       the isopycnal or geopotential s-coord. operator 
   !!                       vector optimization, use k-j-i loops.
   !!----------------------------------------------------------------------

   USE oce             ! ocean dynamics and active tracers
   USE dom_oce         ! ocean space and time domain
   USE ldftra_oce      ! ocean active tracers: lateral physics
   USE trdmod          ! ocean active tracers trends 
   USE trdmod_oce      ! ocean variables trends
   USE zdf_oce         ! ocean vertical physics
   USE in_out_manager  ! I/O manager
   USE ldfslp          ! iso-neutral slopes
   USE diaptr          ! poleward transport diagnostics
   USE prtctl          ! Print control

   IMPLICIT NONE
   PRIVATE

   PUBLIC tra_ldf_iso_grif   ! routine called by traldf.F90

   !! * Substitutions
#  include "domzgr_substitute.h90"
#  include "ldftra_substitute.h90"
#  include "ldfeiv_substitute.h90"
#  include "vectopt_loop_substitute.h90"
   !!----------------------------------------------------------------------
   !!   OPA 9.0 , LOCEAN-IPSL (2005) 
   !! $Header$ 
   !! Software governed by the CeCILL licence (modipsl/doc/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------

CONTAINS

   SUBROUTINE tra_ldf_iso_grif( kt )
      !!----------------------------------------------------------------------
      !!                  ***  ROUTINE tra_ldf_iso_grif  ***
      !!
      !! ** Purpose :   Compute the before horizontal tracer (t & s) diffusive 
      !!      trend for a laplacian tensor (except the dz[ dz[.] ] term) and 
      !!      add it to the general trend of tracer equation.
      !!
      !! ** Method  :   The horizontal component of the lateral diffusive trends 
      !!      is provided by a 2nd order operator rotated along neutral or geopo-
      !!      tential surfaces to which an eddy induced advection can be added
      !!      It is computed using before fields (forward in time)
      !!
      !!      1st part :  masked horizontal derivative of T & S ( di[ t ] )
      !!      ========    with partial cell update if ln_zps=T.
      !!
      !!      2nd part :  horizontal fluxes of the lateral mixing operator
      !!      ========    
      !!      take the horizontal divergence of the fluxes:
      !!         difft = 1/(e1t*e2t*e3t) {  di-1[ zftu ] +  dj-1[ zftv ]  }
      !!      Add this trend to the general trend (ta,sa):
      !!         ta = ta + difft
      !!
      !!      3rd part: vertical trends of the lateral mixing operator
      !!      ========  (excluding the vertical flux proportional to dk[t] )
      !!      vertical fluxes associated with the rotated lateral mixing:

      !!      take the horizontal divergence of the fluxes:
      !!         difft = 1/(e1t*e2t*e3t) dk[ zftw ]
      !!      Add this trend to the general trend (ta,sa):
      !!         ta = ta + difft
      !!
      !! ** Action :
      !!         Update (ta,sa) arrays with the before rotated diffusion trend
      !!            (except the dk[ dk[.] ] term)
      !!
      !!----------------------------------------------------------------------
      USE oce           , zftv => ua   ! use ua as workspace
      USE oce           , zfsv => va   ! use va as workspace
      !!
      INTEGER, INTENT( in ) ::   kt        ! ocean time-step index
      !!
      INTEGER  ::   ji, jj, jk, ip, jp, kp    ! dummy loop indices
      INTEGER  ::   iku, ikv                  ! temporary integer
      REAL(wp) ::   zatempu, zdx, zta         ! temporary scalars
      REAL(wp) ::   zatempv, zdy, zsa         !    "         "
      REAL(wp) ::   zslopec, zdsloper, zepsln !    "         "
      REAL(wp) ::   zsxe, za_sxe, zfact       !    "         "
      REAL(wp) ::   zbtr                      !    "         "
      REAL(wp), DIMENSION(2) ::   zdelta   ! 1D workspace
      REAL(wp), DIMENSION(jpi,jpj)     ::   zftu   ! 2D workspace
      REAL(wp), DIMENSION(jpi,jpj)     ::   zfsu   !    "           "
      REAL(wp), DIMENSION(jpi,jpj,jpk) ::   zdit, zdjt, zdkt     ! 3D workspace
      REAL(wp), DIMENSION(jpi,jpj,jpk) ::   zdis, zdjs, zdks     !  "      "



      !!----------------------------------------------------------------------

      IF( kt == nit000 ) THEN
         IF(lwp) WRITE(numout,*)
         IF(lwp) WRITE(numout,*) 'tra_ldf_iso_grif : rotated laplacian diffusion operator'
         IF(lwp) WRITE(numout,*) '~~~~~~~~~~~~~~~~'
      ENDIF

      IF ( .NOT. lk_traldf_eiv ) THEN
        fsaeiu(:,:,:)=0.0
        fsaeiv(:,:,:)=0.0
        fsaeiw(:,:,:)=0.0
      ENDIF

      DO jk=1,jpkm1
      DO jj=1,jpjm1
      DO ji=1,fs_jpim1
               ftu(ji,jj,jk)=ftud(ji,jj,jk)+ftu(ji,jj,jk)*(fsahtu(ji,jj,jk)-fsaeiu(ji,jj,jk))
               fsu(ji,jj,jk)=fsud(ji,jj,jk)+fsu(ji,jj,jk)*(fsahtu(ji,jj,jk)-fsaeiu(ji,jj,jk))
               ftv(ji,jj,jk)=ftvd(ji,jj,jk)+ftv(ji,jj,jk)*(fsahtv(ji,jj,jk)-fsaeiv(ji,jj,jk))
               fsv(ji,jj,jk)=fsvd(ji,jj,jk)+fsv(ji,jj,jk)*(fsahtv(ji,jj,jk)-fsaeiv(ji,jj,jk))
      END DO
      END DO
      END DO

      DO jk = 1, jpkm1

         ! II.4 Second derivative (divergence) and add to the general trend
         ! ----------------------------------------------------------------
         DO jj = 2 , jpjm1
            DO ji = fs_2, fs_jpim1   ! vector opt.
               zbtr= 1. / ( e1t(ji,jj)*e2t(ji,jj)*fse3t(ji,jj,jk) )
               zta = zbtr * ( ftu(ji,jj,jk) - ftu(ji-1,jj,jk) + ftv(ji,jj,jk) - ftv(ji,jj-1,jk)  )
               zsa = zbtr * ( fsu(ji,jj,jk) - fsu(ji-1,jj,jk) + fsv(ji,jj,jk) - fsv(ji,jj-1,jk)  )
               ta (ji,jj,jk) = ta (ji,jj,jk) + zta
               sa (ji,jj,jk) = sa (ji,jj,jk) + zsa
            END DO
         END DO
         !                                          ! ===============
      END DO                                        !   End of slab  
      !                                             ! ===============

      IF( ln_diaptr .AND. ( MOD( kt, nf_ptr ) == 0 ) ) THEN   ! Poleward diffusive heat and salt transports
         pht_ldf(:) = ptr_vj( zftv(:,:,:) )
         pst_ldf(:) = ptr_vj( zfsv(:,:,:) )
      ENDIF

      !!----------------------------------------------------------------------
      !!   III - vertical trend of T & S (extra diagonal terms only)
      !!----------------------------------------------------------------------

      ! I.5 Divergence of vertical fluxes added to the general tracer trend
      ! -------------------------------------------------------------------

      DO jk=1,jpk
      DO jj=2,jpjm1
      DO ji=fs_2,fs_jpim1
               tfw(ji,jj,jk)=tfw(ji,jj,jk)*(fsahtw(ji,jj,jk)+fsaeiw(ji,jj,jk))
               sfw(ji,jj,jk)=sfw(ji,jj,jk)*(fsahtw(ji,jj,jk)+fsaeiw(ji,jj,jk))
      END DO
      END DO
      END DO

      DO jk = 1, jpkm1
         DO jj = 2, jpjm1
            DO ji = fs_2, fs_jpim1   ! vector opt.
               zbtr =  1. / ( e1t(ji,jj)*e2t(ji,jj)*fse3t(ji,jj,jk) )
               zta  = (  tfw(ji,jj,jk) - tfw(ji,jj,jk+1)  ) * zbtr
               zsa  = (  sfw(ji,jj,jk) - sfw(ji,jj,jk+1)  ) * zbtr
               ta(ji,jj,jk) = ta(ji,jj,jk) + zta
               sa(ji,jj,jk) = sa(ji,jj,jk) + zsa
            END DO
         END DO
      END DO
      !
      DO jk=1,jpk
      DO jj=2,jpjm1
      DO ji=fs_2,fs_jpim1
       psix_eiv(ji,jj,jk) = psix_eiv(ji,jj,jk)*fsaeiu(ji,jj,jk)
       psiy_eiv(ji,jj,jk) = psiy_eiv(ji,jj,jk)*fsaeiv(ji,jj,jk)
      END DO
      END DO
      END DO

   END SUBROUTINE tra_ldf_iso_grif

#else
   !!----------------------------------------------------------------------
   !!   default option :   Dummy code   NO rotation of the diffusive tensor
   !!----------------------------------------------------------------------
CONTAINS
   SUBROUTINE tra_ldf_iso_grif( kt )               ! Empty routine
      WRITE(*,*) 'tra_ldf_iso_grif: You should not have seen this print! error?', kt
   END SUBROUTINE tra_ldf_iso_grif
#endif

   !!==============================================================================
END MODULE traldf_iso_grif
